/*
 * Copyright 2011 Raster Software Vigo
 *
 * This file is part of ttos
 *
 * ttos is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * ttos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * calculator.c
 *
 *  Created on: 16/09/2010
 *      Author: raster
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "../ttos_lib/ttos.h"
#include "../minibus/minibus.h"

struct ttos_object *master;
enum ops {OP_NONE, OP_COPY, OP_ADD, OP_SUB, OP_MUL, OP_DIV} current_op;

struct _value {
	bool negative;
	uint32_t integer;
	uint32_t decimal;
	uint32_t multiplier;
	uint32_t exp;
};

struct _value screen_v,hide,memo,digits;
uint32_t screen;
bool dot;
bool last_was_digit;

void copy_values(struct _value *v1, struct _value *v2) {

	memcpy(v1,v2,sizeof(struct _value));

}

void main_callback(struct ttos_object *object, uint32_t event, void *user_data) {

	struct minibus_data_element *data;
	int32_t error;
	int16_t xoffset,yoffset,ratio;

	if (!master->fg) {
		minibus_exit_loop(object->queue);
		return;
	}
}

void set_zero(struct _value *value) {

	value->integer=0;
	value->decimal=0;
	value->exp=0;
	value->multiplier=1;
	value->negative=false;
	if (value==&digits) {
		dot=false;
	}
}

void set_all_zero() {

	current_op=OP_NONE;
	set_zero(&screen_v);
	set_zero(&hide);
	set_zero(&digits);
}


void print_current(struct _value *value) {

	int8_t string[200],string2[200],*pointer;
	uint32_t loop,size,size2;

	if ((!value->integer)&&(!value->decimal)) { // if it's zero, it must be non-negative
		value->negative=false;
	}

	if (value->negative) {
		size=sprintf(string,"-%u.",value->integer);
	} else {
		size=sprintf(string,"%u.",value->integer);
	}
	pointer=string+size;
	if (value->exp) {
		size2=sprintf(string2,"%u",value->decimal);
		for(loop=size2;loop<value->exp;loop++) {
			*(pointer++)='0';
		}
		*pointer=0;
		strcat(pointer,string2);
	}
	ttos_set_label_text(master,TTOS_SYNC_ANSWER,screen,string,TTOS_WIDGET_FONT_HALIGN_RIGHT|TTOS_WIDGET_FONT_VALIGN_MIDDLE);
	ttos_repaint_screen(master,TTOS_SYNC_ANSWER);

}

void add_value(uint64_t value) {

	if (!dot) {
		digits.integer*=10;
		digits.integer+=value;
	} else {
		digits.decimal*=10;
		digits.decimal+=value;
		digits.exp++;
		digits.multiplier*=10;
	}
	copy_values(&screen_v,&digits);
	print_current(&screen_v);
}

void add_values(struct _value *v1, struct _value *v2) {

	// v1=v1+v2

	uint64_t tv1,tv2,multiplier,m1,m2,total;
	uint32_t exp;

	// first, set both values to the same exp

	if (v1->exp>v2->exp) {
		multiplier=v1->multiplier;
		m1=1;
		m2=v1->multiplier/v2->multiplier;
		exp=v1->exp;
	} else {
		multiplier=v2->multiplier;
		m1=v2->multiplier/v1->multiplier;
		m2=1;
		exp=v2->exp;
	}

	tv1=v1->integer*multiplier+v1->decimal*m1;
	tv2=v2->integer*multiplier+v2->decimal*m2;

	if (v1->negative==v2->negative) { // add both
		total=tv1+tv2;
	} else { // substract them
		if (tv1>tv2) {
			total=tv1-tv2;
		} else {
			total=tv2-tv1;
			v1->negative=v2->negative;
		}
	}

	v1->integer=total/multiplier;
	v1->decimal=total%multiplier;
	v1->multiplier=multiplier;
	v1->exp=exp;

	// adjust the exp
	while((v1->exp)&&(!(v1->decimal%10))) {
		v1->decimal/=10;
		v1->exp--;
		v1->multiplier/=10;
	}
	if ((!v1->integer)&&(!v1->decimal)) { // if it's zero, it must be non-negative
		v1->negative=false;
	}
}

void multiply_values(struct _value *v1, struct _value *v2) {

	// v1*=v2

	uint64_t tv1,tv2,multiplier,m1,m2,total;
	uint32_t exp;

	// first, set both values to the same exp

	if (v1->exp>v2->exp) {
		multiplier=v1->multiplier;
		m1=1;
		m2=v1->multiplier/v2->multiplier;
		exp=v1->exp;
	} else {
		multiplier=v2->multiplier;
		m1=v2->multiplier/v1->multiplier;
		m2=1;
		exp=v2->exp;
	}

	tv1=v1->integer*multiplier+v1->decimal*m1;
	tv2=v2->integer*multiplier+v2->decimal*m2;

	total=tv1*tv2;
	multiplier*=multiplier;
	exp*=2;

	v1->integer=total/multiplier;
	v1->decimal=total%multiplier;
	v1->multiplier=multiplier;
	v1->exp=exp;

	// adjust the exp
	while((v1->exp)&&(!(v1->decimal%10))) {
		v1->decimal/=10;
		v1->exp--;
		v1->multiplier/=10;
	}

	v1->negative = (v1->negative==v2->negative) ? false : true;

	if ((!v1->integer)&&(!v1->decimal)) { // if it's zero, it must be non-negative
		v1->negative=false;
	}
}

void divide_values(struct _value *v1, struct _value *v2) {

	// v1/=v2

	uint64_t tv1,tv2,multiplier,m1,m2,total;
	uint32_t exp,loop,mul2;
	bool exact;

	if ((!v2->integer)&&(!v2->decimal)) { // avoid division by zero
		ttos_set_label_text(master,TTOS_SYNC_ANSWER,screen,"E",TTOS_WIDGET_FONT_HALIGN_RIGHT|TTOS_WIDGET_FONT_VALIGN_MIDDLE);
		ttos_repaint_screen(master,TTOS_SYNC_ANSWER);
		return;
	}

	// first, set both values to the same exp

	if (v1->exp>v2->exp) {
		multiplier=v1->multiplier;
		m1=1;
		m2=v1->multiplier/v2->multiplier;
		exp=v1->exp;
	} else {
		multiplier=v2->multiplier;
		m1=v2->multiplier/v1->multiplier;
		m2=1;
		exp=v2->exp;
	}

	tv2=v2->integer*multiplier+v2->decimal*m2;
	tv1=v1->integer*multiplier+v1->decimal*m1;
	exp=0;
	multiplier=1;
	exact=false;
	for(loop=0;loop<12;loop++) {
		total=tv1/tv2;
		if (total*tv2==tv1) {
			exact=true;
			break;
		}
		tv1*=10;
		multiplier*=10;
		exp++;
	}

	if (!exact) {
		multiplier/=10;
		exp--;
	}
	m1=(total/multiplier);
	m2=(total%multiplier);
	while(m2>0x0FFFFFFFF) {
		m2/=10;
		multiplier/=10;
		exp--;
	}
	v1->integer=(uint32_t)m1;
	v1->decimal=(uint32_t)m2;
	v1->multiplier=multiplier;
	v1->exp=exp;

	// adjust the exp
	while((v1->exp)&&(!(v1->decimal%10))) {
		v1->decimal/=10;
		v1->exp--;
		v1->multiplier/=10;
	}

	v1->negative = (v1->negative==v2->negative) ? false : true;

	if ((!v1->integer)&&(!v1->decimal)) { // if it's zero, it must be non-negative
		v1->negative=false;
	}
}

void invert_sign(struct _value *v) {

	if ((v->decimal)||(v->integer)) {
		v->negative = v->negative ? false : true;
	} else {
		v->negative=false;
	}
}

void do_op(enum ops next_op) {

	switch(current_op) {
	case OP_ADD:
		add_values(&hide,&screen_v);
	break;
	case OP_SUB:
		invert_sign(&screen_v);
		add_values(&hide,&screen_v);
	break;
	case OP_MUL:
		multiply_values(&hide,&screen_v);
	break;
	case OP_DIV:
		divide_values(&hide,&screen_v);
	break;
	case OP_NONE:
		copy_values(&hide,&screen_v);
	break;
	}

	set_zero(&digits);
	copy_values(&screen_v,&hide);
	print_current(&screen_v);
	current_op=next_op;
}

uint32_t default_callback(uint32_t port,uint32_t answer_port, struct minibus_connection *conn, void *user_data) {

	switch(port) {
	case 10: // off
		exit(0);
	break;
	case 11: // AC
		set_all_zero();
		print_current(&screen_v);
		last_was_digit=false;
	break;
	case 12: // =
		do_op(OP_NONE);
		last_was_digit=false;
	break;
	case 100:
	case 101:
	case 102:
	case 103:
	case 104:
	case 105:
	case 106:
	case 107:
	case 108:
	case 109:
		add_value(port-100);
		last_was_digit=true;
	break;
	case 110: // .
		dot=true;
		last_was_digit=true;
	break;
	case 130: // +
		do_op(OP_ADD);
		last_was_digit=false;
	break;
	case 131: // -
		do_op(OP_SUB);
		last_was_digit=false;
	break;
	case 132: // *
		do_op(OP_MUL);
		last_was_digit=false;
	break;
	case 133: // *
		do_op(OP_DIV);
		last_was_digit=false;
	break;
	case 134: // +/-
		if (last_was_digit) {
			invert_sign(&digits);
			copy_values(&screen_v,&digits);
		} else {
			invert_sign(&hide);
			copy_values(&screen_v,&hide);
		}
		print_current(&screen_v);
	break;
	case 121: //M-
		invert_sign(&screen_v);
	case 120: //M+
		add_values(&memo,&screen_v);
		last_was_digit=false;
		if(port==121) {
			invert_sign(&screen_v);
		}
		set_zero(&digits);
	break;
	case 122: // MC
		set_zero(&memo);
		set_zero(&digits);
		last_was_digit=false;
	break;
	case 123: // MR
		copy_values(&screen_v,&memo);
		copy_values(&digits,&memo);
		do_op(OP_NONE);
		//print_current(&screen_v);
		last_was_digit=false;
	break;
	}
}

void paint_calc() {

	struct _ttos_key_buttons buttons[]={
		{TTOS_NO_STOCK_BUTTON,"M+",NULL,true,120,1,0},
		{TTOS_NO_STOCK_BUTTON,"7" ,NULL,true,107,6,0},
		{TTOS_NO_STOCK_BUTTON,"8" ,NULL,true,108,6,0},
		{TTOS_NO_STOCK_BUTTON,"9" ,NULL,true,109,6,0},
		{TTOS_NO_STOCK_BUTTON,"+" ,NULL,true,130,2,0},
		{TTOS_EXIT_BUTTON,    NULL,NULL,false,10,3,0},
		{TTOS_NO_STOCK_BUTTON,"M-",NULL,true,121,1,0},
		{TTOS_NO_STOCK_BUTTON,"4" ,NULL,true,104,6,0},
		{TTOS_NO_STOCK_BUTTON,"5" ,NULL,true,105,6,0},
		{TTOS_NO_STOCK_BUTTON,"6" ,NULL,true,106,6,0},
		{TTOS_NO_STOCK_BUTTON,"-" ,NULL,true,131,2,0},
		{TTOS_NO_STOCK_BUTTON,"AC",NULL,true, 11,4,0},
		{TTOS_NO_STOCK_BUTTON,"MC",NULL,true,122,1,0},
		{TTOS_NO_STOCK_BUTTON,"1" ,NULL,true,101,6,0},
		{TTOS_NO_STOCK_BUTTON,"2" ,NULL,true,102,6,0},
		{TTOS_NO_STOCK_BUTTON,"3" ,NULL,true,103,6,0},
		{TTOS_NO_STOCK_BUTTON,"x" ,NULL,true,132,2,0},
		{TTOS_NO_STOCK_BUTTON,"=" ,NULL,true, 12,5,2},
		{TTOS_NO_STOCK_BUTTON,"MR",NULL,true,123,1,0},
		{TTOS_NO_STOCK_BUTTON,"0" ,NULL,true,100,6,0},
		{TTOS_NO_STOCK_BUTTON,"." ,NULL,true,110,7,0},
		{TTOS_NO_STOCK_BUTTON,"+/-",NULL,true,134,2,0},
		{TTOS_NO_STOCK_BUTTON,"/" ,NULL,true,133,2,0},
		{TTOS_NO_STOCK_BUTTON,NULL,NULL,false,0,0,0}
	};
	struct ttos_color color[]={
			{0xEF,0xEF,0xEF,0xFF}, // 0 background color
			{0x40,0xFF,0x40,0xFF}, // 1 memory ops color
			{0xFF,0xFF,0x40,0xFF}, // 2 main ops color
			{0xFF,0x40,0x40,0xFF}, // 3 exit color
			{0xFF,0x40,0x40,0xFF}, // 4 clear color
			{0xFF,0xFF,0xFF,0xFF}, // 5 = color
			{0xFF,0xFF,0xFF,0xFF}, // 6 numbers color
			{0xFF,0xFF,0xFF,0xFF}, // 7 dot color
			{0xBF,0xBF,0xBF,0xFF}, // 8 screen color
			{0x00,0x00,0x00,0xFF}  // 9 screen numbers color
	};
	struct ttos_keyboard keyboard;

	uint32_t key_w,key_h;
	uint32_t newtab;
	int16_t margin,y;

	margin=5;
	y=50;

	newtab=ttos_new_tab(master,TTOS_SYNC_ANSWER);
	ttos_set_tab_bg(master,TTOS_SYNC_ANSWER,newtab,color);

	ttos_new_color_area(master,TTOS_SYNC_ANSWER,newtab,margin,2*margin,master->width-2*margin,y-4*margin,&color[8],0,0);
	screen=ttos_new_label(master,TTOS_SYNC_ANSWER,newtab,2*margin,2*margin,master->width-4*margin,y-4*margin,"",&color[9],TTOS_WIDGET_FONT_HALIGN_RIGHT|TTOS_WIDGET_FONT_VALIGN_MIDDLE,0);

	keyboard.buttons=buttons;
	keyboard.colors=color;
	keyboard.width=6;
	keyboard.height=4;
	ttos_paint_keyboard(master,newtab,margin,y,master->width-2*margin,master->height-y-margin,margin,margin,&keyboard,false);

	ttos_set_visible_tab(master,TTOS_SYNC_ANSWER,newtab,TTOS_TAB_SWAP_MODE_REPLACE, TTOS_NO_ANSWER);
	print_current(&screen_v);
}

int main(int argc,char **argv) {

	struct minibus_connection *conn;
	struct ttos_color tcolor,bgcolor;
	uint32_t timeto;

	master=ttos_init_system(0,argc,argv);

	if (master==NULL) {
		printf("Can't connect to the queue. Aborting\n");
		exit(1);
	}

	master->cb_port0=main_callback;
	minibus_set_default_callback(master->queue,default_callback,NULL);

	set_all_zero();
	set_zero(&memo);

	paint_calc();

	minibus_run_loop(master->queue);

	return 0;
}
