/*
 * Copyright 2011 Raster Software Vigo
 *
 * This file is part of ttos
 *
 * ttos is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * ttos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * global_vars.h
 *
 *  Created on: 16/09/2010
 *      Author: raster
 */

#ifndef GLOBAL_VARS_H_
#define GLOBAL_VARS_H_


#include "minibus/minibus.h"
#include "blit.h"
#include "fonts.h"
#include "uid.h"

/**
 * This structure maintains all the data about a client connected to ttos
 */

struct _client_data {

	struct minibus_connection *conn; ///< Client's connection
	uint32_t uid;
	Cstring app_name;
	uint8_t window; ///< Window associated to this client
	bool window_manager; ///< True if this client is the window manager
	struct minibus_list tabs; ///< List with all the tabs defined by this client
	struct minibus_list sharemem; ///< List with all the shared memory blocks created by this client
	struct _tab *show_tab; ///< The tab currently being shown by this client
	struct _tab *edit_tab; ///< A cache for the tab being edited by this client
	uint32_t ipc_uid; ///< The IPC uid for receiving messages
	void *data; ///< Private data
};


/**
 * This structure defines each possible window in screen
 */

struct _window {
	uint16_t ox; ///< Minimum X value
	uint16_t oy; ///< Minimum Y value
	uint16_t mx; ///< Maximum X value
	uint16_t my; ///< Maximum Y value
	uint16_t w; ///< Window width
	uint16_t h; ///< Window height
	bool visible; ///< If this window will be painted or not
	struct _client_data *fg_client; ///< Client which is in foreground in this window
};

extern struct _window *windows;
uint32_t n_windows; ///< Number of defined windows

extern struct minibus_master *main_queue;
extern struct minibus_list clients;
extern struct minibus_list bitmaps;
extern struct minibus_list timers;
extern struct minibus_list tab_scroll;
extern struct minibus_list ipc;
extern struct _client_data *window_manager;
extern struct minibus_connection *write_conn;

extern TTOS_Surface *base;
extern struct tsdev *ts;

extern uint32_t touch_window;
extern struct _widget *touch_widget;

extern uint32_t touch_counter;
extern bool large_touch_made;

extern struct _font_base *base_font;

extern struct ttos_uid_c *ttos_uids;
extern struct ttos_uid_c *ttos_ipc_uid;

int32_t main_refresh(struct minibus_connection *connection);

#endif /* GLOBAL_VARS_H_ */
