/*
 * Copyright 2010 (C) Raster Software Vigo (Sergio Costas)
 * This file is part of MINIBUS
 *
 * MINIBUS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * MINIBUS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/*
 * minibus_other.c
 *
 *  Created on: 27/01/2011
 *      Author: raster
 */

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "minibus.h"

/**
 * Appends a path to another path
 *
 * @param path1 Base path
 * @param path2 Append path
 * @return path2 appended to path1 (unless path2 starts with '/'; in that case, will return path2)
 */

Cstring minibus_path_join(Cstring path1,Cstring path2) {

	Cstring final_path;
	uint32_t size1,size2;

	if (!path1) {
		if (!path2) {
			return NULL;
		} else {
			return(strdup(path2));
		}
	}

	if(!path2) {
		return(strdup(path1));
	}

	if (*path2=='/') {
		return(strdup(path2));
	}

	size1=strlen(path1);
	size2=strlen(path2);
	if (!size1) {
		final_path=strdup(path2);
		size1=size2;
	} else {
		if (path1[size1-1]=='/') {
			size1+=1+size2;
			final_path=malloc(size1);
			if (!final_path) {
				return NULL;
			}
			sprintf(final_path,"%s%s",path1,path2);
		} else {
			size1+=2+size2;
			final_path=malloc(size1);
			if (!final_path) {
				return NULL;
			}
			sprintf(final_path,"%s/%s",path1,path2);
		}
	}

	if ((size1>1)&&(final_path[size1-1]=='/')) {
		final_path[size1-1]=0;
	}
	return (final_path);
}

Dstring minibus_new_dstring(char *text) {

	Dstring object;

	object=malloc(sizeof (struct _dstring));
	if (!object) {
		return NULL;
	}
	object->text=strdup(text);
	object->size=strlen(text);
	object->allocated=1+object->size;

	return (object);
}

void minibus_destroy_dstring(Dstring data, bool free_text) {

	if (!data) {
		return;
	}
	if (free_text) {
		free(data->text);
	}
	free(data);
}

void minibus_append_dstring(Dstring data, char *text) {

	size_t text_length,new_size;

	text_length=strlen(text);

	if ((data->allocated)<=(text_length+data->size)) {
		new_size=(1+((text_length+data->size)>>8))<<8;
		data->text=realloc(data->text,new_size);
		data->allocated=new_size;
	}
	memcpy(data->text+data->size,text,text_length);
	data->size+=text_length;
	*(data->text+data->size)=0;
}

void minibus_cut_dstring(Dstring data, size_t to_cut) {

	if (to_cut>data->size) {
		to_cut=data->size;
	}
	data->size-=to_cut;
	*(data->text+data->size)=0;
}
