/*
 * Copyright 2011 Raster Software Vigo
 *
 * This file is part of ttos
 *
 * ttos is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * ttos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * player.c
 *
 *  Created on: 16/09/2010
 *      Author: raster
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <locale.h>
#include <libintl.h>
#include <sys/types.h>
#include <dirent.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <sys/wait.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>

#define _(string) gettext (string)

extern char **environ;

#include "../ttos_lib/ttos.h"
#include "../minibus/minibus.h"

#include "player.h"

struct ttos_object *master;
uint32_t tab_id;
uint32_t info_label;
uint32_t play_button;
uint32_t pause_button;
bool main_iface;
Cstring songs_path;
struct minibus_list songlist,albumlist;
int childpid=0;
struct s_song *current_song;
struct s_album *current_album;
enum e_play_mode play_mode;
enum e_play_status play_status;
enum e_current_iface current_iface;

void list_songs(char modo) {

	struct minibus_list_element *l1,*l2;
	struct s_song *s;
	struct s_album *a;
	uint32_t counter;

	if(modo==2) {
		for(l1=songlist.list;l1;l1=l1->next) {
			s=l1->data;
			printf("%s %s\n",s->album->path,s->filename);
		}
		return;
	}

	for(l1=albumlist.list;l1;l1=l1->next) {
		a=l1->data;
		printf("Album %s (%d songs)\n",a->path,a->count);
		for(l2=songlist.list;l2;l2=l2->next) {
			s=l2->data;
			if(s==a->first_song) {
				break;
			}
		}
		if(!l2) {
			printf("No encontrada cancion\n");
			continue;
		}
		for(counter=0;counter<a->count;counter++) {
			s=l2->data;
			printf("   Cancion: %s ",s->filename);
			if (s->album!=a) {
				printf("Error, album incorrecto");
			}
			printf("\n");
			l2=l2->next;
			if (!l2) {
				printf("Fin de lista\n");
				break;
			}
		}
	}
}


void list_songs2() {

	struct minibus_list_element *element;
	struct s_song *song;

	for(element=songlist.list;element;element=element->next) {
		song=element->data;
		printf("Cancion %s album %s\n",song->filename,song->album->path);
	}
}


void destroy_album(void *object) {

	struct s_album *c_album;

	if (!object) {
		return;
	}
	c_album=object;
	free(c_album->path);
	free(c_album);
}


void destroy_song(void *object) {

	struct s_song *c_song;

	if (!object) {
		return;
	}
	c_song=object;
	free(c_song->filename);
	free(c_song);
}


struct s_album *create_album(Cstring album_path) {

	struct s_album *new_album;

	new_album=malloc(sizeof(struct s_album));
	if (!new_album) {
		return NULL;
	}
	new_album->path=strdup(album_path);
	new_album->count=0;
	new_album->first_song=NULL;
	new_album->last_song=NULL;
	return (new_album);
}


struct s_song *create_song(Cstring song_filename,struct s_album *c_album) {

	struct s_song *obj;

	obj=malloc(sizeof(struct s_song));
	if (!obj) {
		return NULL;
	}
	switch (ttos_check_extension(song_filename)) {
	case TTOS_FTYPE_MP3:
		obj->type=SONG_MP3;
	break;
	case TTOS_FTYPE_OGG:
	case TTOS_FTYPE_VORBIS:
		obj->type=SONG_OGG;
	break;
	default:
		obj->type=SONG_OTHER;
	break;
	}
	obj->filename=strdup(song_filename);
	obj->album=c_album;
	return (obj);
}


bool sort_song(void *v1, void *v2) {

	struct s_song *song1,*song2;

	song1=v1;
	song2=v2;

	if (song1->album!=song2->album) {
		return ((song1>song2) ? true : false);
	}
	if (strcmp(song1->filename,song2->filename)<=0) {
		return true;
	} else {
		return false;
	}
}


int32_t fill_album(Cstring f_path,struct minibus_list *to_add) {

	DIR *directory;
	Cstring path;
	struct dirent *entry;
	struct s_album *new_album;
	struct s_song *new_song;
	struct minibus_list_element *element;
	struct minibus_list minilist;
	bool found_songs;
	uint32_t counter;

	new_album=create_album(f_path);
	if(!new_album) {
		return -4;
	}

	minibus_list_init(&minilist,destroy_song,sort_song);

	directory=opendir(new_album->path);
	if (!directory) {
		destroy_album(new_album);
		return -1;
	}

	found_songs=false;
	counter=0;
	while(NULL!=(entry=readdir(directory))) {

		if ((!strcmp(entry->d_name,"."))||(!strcmp(entry->d_name,".."))) {
			continue;
		}

		if(entry->d_type==DT_DIR) {
			path=minibus_path_join(new_album->path,entry->d_name);
			minibus_list_add(to_add,path);
			continue;
		}

		if(entry->d_type==DT_REG) {
			new_song=create_song(entry->d_name,new_album);
			if (!new_song) {
				closedir(directory);
				destroy_album(new_album);
				minibus_list_free(&minilist);
				return -3;
			}
			if (new_song->type==SONG_OTHER) {
				destroy_song(new_song);
				continue;
			}
			found_songs=true;
			element=minibus_list_add_sorted(&minilist,new_song);
			counter++;
			if ((!element->prev)||(((struct s_song *)element->prev->data)->album!=new_album)) {
				new_album->first_song=new_song;
			}
			if ((!element->next)||(((struct s_song *)element->next->data)->album!=new_album)) {
				new_album->last_song=new_song;
			}
		}
	}
	if (found_songs) { // there is, at least, one song
		minibus_list_add(&albumlist,new_album);
		new_album->count=counter;
		minibus_list_append(&songlist,&minilist);
	} else {
		destroy_album(new_album);
	}
	closedir(directory);
	return 0;
}


void destroy_todo(void *object) {

	if (!object) {
		return;
	}
	free(object); // it's just a Cstring, so we can free it directly
}


void killchild() {

	if(!childpid) {
		return;
	}

	kill(childpid,SIGKILL);
	childpid=0;

}


void create_album_list(Cstring base_path) {

	struct minibus_list to_do;
	Cstring path;

	minibus_list_free(&albumlist);
	minibus_list_free(&songlist);
	minibus_list_init(&to_do,destroy_todo,NULL);
	minibus_list_add(&to_do,strdup(base_path)); //this is the first path to add

	while(to_do.list) { // while there are elements in the list...
		path=to_do.list->data;
		minibus_list_remove(to_do.list,false);
		fill_album(path,&to_do);
		free(path);
	}

	minibus_list_free(&to_do);
}


void repaint_interface() {

	switch(current_iface) {
	case IFACE_MAIN:
		paint_main_menu();
	break;
	case IFACE_PLAY:
		paint_iface();
	break;
	}

}


uint32_t paint_main_menu() {

	uint32_t new_tab;
	uint16_t incy;
	struct ttos_color color;

	killchild();
	main_iface=false;
	play_status=PLAY_STOPPED;

	color.r=color.g=color.b=color.a=0xFF;

	//incx=(master->width-320)/2;
	incy=(master->height-240)/2;

	new_tab=ttos_new_tab(master,TTOS_SYNC_ANSWER);
	ttos_set_tab_bg(master,TTOS_SYNC_ANSWER,new_tab,&color);

	ttos_new_label(master,TTOS_SYNC_ANSWER,new_tab,5,5+incy,master->width-10,42,"TTOS music player",NULL,TTOS_WIDGET_FONT_VALIGN_MIDDLE|TTOS_WIDGET_FONT_HALIGN_CENTER,0);
	ttos_new_button(master,TTOS_SYNC_ANSWER,new_tab,5,52+incy,master->width-10,42,TTOS_NO_STOCK_BUTTON,NULL,_("Songs shuffle"),NULL,NULL,TTOS_WIDGET_FONT_HALIGN_CENTER|TTOS_WIDGET_FONT_VALIGN_MIDDLE,10);
	ttos_new_button(master,TTOS_SYNC_ANSWER,new_tab,5,99+incy,master->width-10,42,TTOS_NO_STOCK_BUTTON,NULL,_("Albums shuffle"),NULL,NULL,TTOS_WIDGET_FONT_HALIGN_CENTER|TTOS_WIDGET_FONT_VALIGN_MIDDLE,11);
	ttos_new_button(master,TTOS_SYNC_ANSWER,new_tab,5,146+incy,master->width-10,42,TTOS_NO_STOCK_BUTTON,NULL,_("Play song"),NULL,NULL,TTOS_WIDGET_FONT_HALIGN_CENTER|TTOS_WIDGET_FONT_VALIGN_MIDDLE,12);
	ttos_new_button(master,TTOS_SYNC_ANSWER,new_tab,5,193+incy,master->width-10,42,TTOS_EXIT_BUTTON,NULL,NULL,NULL,NULL,TTOS_WIDGET_FONT_HALIGN_CENTER|TTOS_WIDGET_FONT_VALIGN_MIDDLE,13);

	if (TTOS_ERROR_NO_ERROR==ttos_set_visible_tab(master,TTOS_SYNC_ANSWER,new_tab,TTOS_TAB_SWAP_MODE_SCROLL_LEFT, 26)) {
		minibus_wait_for_port(master->conn,26,false);
	}

	if (tab_id) {
		ttos_destroy_tab(master,TTOS_SYNC_ANSWER,tab_id,0);
	}
	ttos_repaint_screen(master,TTOS_SYNC_ANSWER);
	tab_id=new_tab;
	return 0;
}


void cut_string(Cstring str, uint32_t size) {

	uint32_t counter;

	str[size]=0;
	str+=size;
	for(counter=0;(*str<=' ')&&(counter<size);counter++,str--);
	str++;
	*str=0;
}


int32_t play_file(struct s_song *c_song) {

	char *p1[3];
	Cstring filename;
	int file;
	uint8_t b1[128],b2[227];
	Cstring song,author,album,final,tmp;
	bool v2;
	uint32_t s1,s2,s3;

	killchild();

	filename=minibus_path_join(c_song->album->path,c_song->filename);

	file=open(filename,O_RDONLY);
	if (file<0) {
		return -1;
	}
	lseek(file,-128,SEEK_END);
	read(file,b1,128);
	lseek(file,-355,SEEK_END);
	read(file,b2,227);
	close(file);

	if (!strncmp("TAG",(char *)b1,3)) {
		if (!strncmp("TAG+",(char *)b2,4)) {
			v2=true;
			song=malloc(91);
			author=malloc(91);
			album=malloc(91);
		} else {
			v2=false;
			song=malloc(31);
			author=malloc(31);
			album=malloc(31);
		}
		memcpy(song,b1+3,30);
		memcpy(author,b1+33,30);
		memcpy(album,b1+63,30);
		if(v2) {
			memcpy(song+30,b2+4,60);
			memcpy(author+30,b2+64,60);
			memcpy(album+30,b2+124,60);
			cut_string(song,90);
			cut_string(author,90);
			cut_string(album,90);
		} else {
			cut_string(song,30);
			cut_string(author,30);
			cut_string(album,30);
		}
	} else {
		author=strdup("");
		song=strdup(c_song->filename);
		free(ttos_get_upper_path(c_song->album->path,&tmp));
		album=strdup(tmp);
	}

	s1=strlen(song);
	s2=strlen(author);
	s3=strlen(album);
	final=malloc(s1+s2+s3+5);

	if (s2) {
		sprintf(final,"%s\n\n%s\n\n%s",song,author,album);
	} else {
		sprintf(final,"%s\n\n%s",song,album);
	}

	ttos_set_label_text(master,TTOS_SYNC_ANSWER,info_label,final,TTOS_WIDGET_FONT_HALIGN_CENTER|TTOS_WIDGET_FONT_MULTILINE|TTOS_WIDGET_FONT_WORD_CUT|TTOS_WIDGET_FONT_LATIN1);
	ttos_repaint_screen(master,TTOS_SYNC_ANSWER);

	free(song);
	free(album);
	free(author);
	free(final);

	switch(c_song->type) {
	case SONG_MP3:
		p1[0]="mpg123";
	break;
	case SONG_OGG:
		p1[0]="ogg123";
	break;
	default:
		return -1;
	}
	p1[1]=filename;
	p1[2]=NULL;

	printf("Lanzamos hijo\n");

	childpid=ttos_launch_child(p1,NULL);
	printf("Hijo lanzado\n");
	free(filename);
	return 0;
}


void set_buttons() {

	switch(play_status) {
	case PLAY_PAUSED:
		ttos_set_visibility(master,TTOS_SYNC_ANSWER,play_button,true);
		ttos_set_visibility(master,TTOS_SYNC_ANSWER,pause_button,false);
	break;
	case PLAY_PLAYING:
		ttos_set_visibility(master,TTOS_SYNC_ANSWER,play_button,false);
		ttos_set_visibility(master,TTOS_SYNC_ANSWER,pause_button,true);
	break;
	default:
	break;
	}
	ttos_repaint_screen(master,TTOS_SYNC_ANSWER);

}


uint32_t play_pause_callback(uint32_t port, uint32_t answer_port, struct minibus_connection *conn,void *user_data) {

	switch(play_status) {
	case PLAY_PLAYING:
		play_status=PLAY_PAUSED;
		set_buttons();
		kill(childpid,SIGUSR1);
	break;
	case PLAY_PAUSED:
		play_status=PLAY_PLAYING;
		set_buttons();
		kill(childpid,SIGUSR2);
	break;
	default:
	break;
	}

	return 0;
}


uint32_t next_song_callback(uint32_t port, uint32_t answer_port, struct minibus_connection *conn,void *user_data) {

	play_status=PLAY_PLAYING;
	set_buttons();
	play_next_song();

	return 0;

}


uint32_t prev_song_callback(uint32_t port, uint32_t answer_port, struct minibus_connection *conn,void *user_data) {

	play_status=PLAY_PLAYING;
	set_buttons();
	play_prev_song();

	return 0;
}

uint32_t next_album_callback(uint32_t port, uint32_t answer_port, struct minibus_connection *conn,void *user_data) {

	play_status=PLAY_PLAYING;
	set_buttons();
	play_next_album();

	return 0;

}


uint32_t prev_album_callback(uint32_t port, uint32_t answer_port, struct minibus_connection *conn,void *user_data) {

	play_status=PLAY_PLAYING;
	set_buttons();
	play_prev_album();

	return 0;
}

void paint_iface() {

	uint32_t new_tab;
	uint16_t incx,incy;
	struct ttos_color color;

	if (main_iface) {
		return;
	}
	main_iface=true;

	color.r=color.g=color.b=color.a=0xFF;

	incx=(master->width-320)/2;
	incy=(master->height-240)/2;

	new_tab=ttos_new_tab(master,TTOS_SYNC_ANSWER);
	ttos_set_tab_bg(master,TTOS_SYNC_ANSWER,new_tab,&color);

	ttos_new_label(master,TTOS_SYNC_ANSWER,new_tab,5,10+incy,master->width-10,15,"TTOS Media Player",NULL,TTOS_WIDGET_FONT_VALIGN_MIDDLE|TTOS_WIDGET_FONT_HALIGN_CENTER,0);
	info_label=ttos_new_label(master,TTOS_SYNC_ANSWER,new_tab,15,40+incy,master->width-30,125,"",NULL,TTOS_WIDGET_FONT_VALIGN_MIDDLE|TTOS_WIDGET_FONT_HALIGN_CENTER,0);

	ttos_new_button(master,TTOS_SYNC_ANSWER,new_tab,master->width-34,5+incy,34,34,TTOS_EXIT_BUTTON,NULL,NULL,NULL,NULL,TTOS_WIDGET_FONT_HALIGN_CENTER|TTOS_WIDGET_FONT_VALIGN_MIDDLE|TTOS_WIDGET_BUTTON_NO_TEXT,20);

	play_button=ttos_new_button(master,TTOS_SYNC_ANSWER,new_tab,130+incx,170+incy,60,60,TTOS_NO_STOCK_BUTTON,"player/play.tga",NULL,NULL,NULL,TTOS_WIDGET_FONT_HALIGN_CENTER|TTOS_WIDGET_FONT_VALIGN_MIDDLE,21);
	pause_button=ttos_new_button(master,TTOS_SYNC_ANSWER,new_tab,130+incx,170+incy,60,60,TTOS_NO_STOCK_BUTTON,"player/pause.tga",NULL,NULL,NULL,TTOS_WIDGET_FONT_HALIGN_CENTER|TTOS_WIDGET_FONT_VALIGN_MIDDLE,21);
	ttos_set_visibility(master,TTOS_SYNC_ANSWER,play_button,false);
	ttos_set_visibility(master,TTOS_SYNC_ANSWER,pause_button,true);

	ttos_new_button(master,TTOS_SYNC_ANSWER,new_tab,65+incx,170+incy,60,60,TTOS_NO_STOCK_BUTTON,"player/prev_song.tga",NULL,NULL,NULL,TTOS_WIDGET_FONT_HALIGN_CENTER|TTOS_WIDGET_FONT_VALIGN_MIDDLE,22);
	ttos_new_button(master,TTOS_SYNC_ANSWER,new_tab,195+incx,170+incy,60,60,TTOS_NO_STOCK_BUTTON,"player/next_song.tga",NULL,NULL,NULL,TTOS_WIDGET_FONT_HALIGN_CENTER|TTOS_WIDGET_FONT_VALIGN_MIDDLE,23);

	if (play_mode==PLAY_TITLES) {
		ttos_new_button(master,TTOS_SYNC_ANSWER,new_tab,incx,170+incy,60,60,TTOS_NO_STOCK_BUTTON,"player/prev_album.tga",NULL,NULL,NULL,TTOS_WIDGET_FONT_HALIGN_CENTER|TTOS_WIDGET_FONT_VALIGN_MIDDLE,24);
		ttos_new_button(master,TTOS_SYNC_ANSWER,new_tab,260+incx,170+incy,60,60,TTOS_NO_STOCK_BUTTON,"player/next_album.tga",NULL,NULL,NULL,TTOS_WIDGET_FONT_HALIGN_CENTER|TTOS_WIDGET_FONT_VALIGN_MIDDLE,25);
	}

	if (TTOS_ERROR_NO_ERROR==ttos_set_visible_tab(master,TTOS_SYNC_ANSWER,new_tab,TTOS_TAB_SWAP_MODE_SCROLL_RIGHT, 26)) {
		minibus_wait_for_port(master->conn,26,false);
	}

	if (tab_id) {
		ttos_destroy_tab(master,TTOS_SYNC_ANSWER,tab_id,0);
	}
	ttos_repaint_screen(master,TTOS_SYNC_ANSWER);
	tab_id=new_tab;

}

void play_music() {

	play_status=PLAY_PLAYING;
	current_iface=IFACE_PLAY;
	repaint_interface();

	play_file(current_song);


}


uint32_t paint_main_menu_callback(uint32_t port, uint32_t answer_port, struct minibus_connection *conn,void *user_data) {

	current_iface=IFACE_MAIN;
	repaint_interface();

	return 0;
}


uint32_t shuffle_songs_callback(uint32_t port, uint32_t answer_port, struct minibus_connection *conn,void *user_data) {

	uint64_t counter;
	uint32_t randval,loop;
	struct minibus_list_element *element;
	struct minibus_list tmp_list;

	create_album_list(songs_path);

	minibus_list_init(&tmp_list,destroy_song,NULL);
	minibus_list_append(&tmp_list,&songlist);

	counter=0;
	for(element=tmp_list.list,counter=0;element;element=element->next,counter++);
	while(counter) {
		randval=(uint32_t)((((uint64_t)random())*counter)/((uint64_t)RAND_MAX));
		element=tmp_list.list;
		for(loop=0;loop<randval;loop++) {
			element=element->next;
		}
		minibus_list_add(&songlist,element->data);
		minibus_list_remove(element,false);
		counter--;
	}

	play_mode=PLAY_SONGS;
	current_song=songlist.list->data;
	current_album=NULL;

	play_music();

	return 0;
}


uint32_t shuffle_albums_callback(uint32_t port, uint32_t answer_port, struct minibus_connection *conn,void *user_data) {

	uint64_t counter;
	uint32_t randval,loop;
	struct minibus_list_element *element;
	struct minibus_list tmp_list;

	create_album_list(songs_path);

	minibus_list_init(&tmp_list,destroy_album,NULL);
	minibus_list_append(&tmp_list,&albumlist);

	counter=0;
	for(element=tmp_list.list,counter=0;element;element=element->next,counter++);
	while(counter) {
		randval=(uint32_t)((((uint64_t)random())*counter)/((uint64_t)RAND_MAX));
		element=tmp_list.list;
		for(loop=0;loop<randval;loop++) {
			element=element->next;
		}
		minibus_list_add(&albumlist,element->data);
		minibus_list_remove(element,false);
		counter--;
	}

	play_mode=PLAY_TITLES;

	current_album=albumlist.list->data;
	current_song=current_album->first_song;

	play_music();

	return 0;
}


uint32_t play_song_callback(uint32_t port, uint32_t answer_port, struct minibus_connection *conn,void *user_data) {

	Cstring path;
	Cstring nfile,pfile;
	struct minibus_list_element *element;
	struct s_album *album;
	struct s_song *song;
	struct ttos_dialog_button buttons = {TTOS_OK_BUTTON , NULL, NULL, 0};
	enum ttos_file_type filetypes[]={TTOS_FTYPE_MP3, TTOS_FTYPE_VORBIS, TTOS_FTYPE_OGG, TTOS_FTYPE_END_LIST};

	path=ttos_select_file(master,songs_path,0,filetypes);
	if (!path) {
		repaint_interface();
		return 0;
	}
	create_album_list(songs_path);
	pfile=ttos_get_upper_path(path,&nfile); // returns the filename in nfile, and its path in pfile
	for(element=albumlist.list;element;element=element->next) {
		album=element->data;
		if (!strcmp(album->path,pfile)) {
			break;
		}
		album=NULL;
	}

	free(pfile);
	if (!album) {
		free(path);
		ttos_show_dialog(master,_("The file seems not to be a valid music file."),TTOS_WIDGET_FONT_ALIGN_CENTER,&buttons,1);
		repaint_interface();
		return 0;
	}

	for(element=minibus_find_data(&songlist,album->first_song);element;element=element->next) {
		song=element->data;
		if (song->album!=album) {
			song=NULL;
			break;
		}
		if (!strcmp(song->filename,nfile)) {
			break;
		}
		song=NULL;
	}

	if (!song) {
		free(path);
		ttos_show_dialog(master,_("The file seems not to be a valid music file."),TTOS_WIDGET_FONT_ALIGN_CENTER,&buttons,1);
		repaint_interface();
		return 0;
	}

	free(path);
	play_mode=PLAY_TITLES;
	current_song=song;
	current_album=song->album;
	play_music();

	return 0;
}


uint32_t exit_callback(uint32_t port, uint32_t answer_port, struct minibus_connection *conn,void *user_data) {

	printf("Salgo PLAYER\n");
	killchild();
	minibus_exit_loop(master->conn->master_queue);

	return 0;
}


void play_next_album() {

	struct minibus_list_element *element;

	element=minibus_find_data(&albumlist,current_song->album);
	while(1) {
		if (!element) {
			printf("Next album not found\n");
			current_iface=IFACE_MAIN;
			repaint_interface();
			return;
		}
		if (!element->next) {
			current_iface=IFACE_MAIN;
			repaint_interface();
			return;
		}
		current_album=element->next->data;
		if (!current_album->first_song) {
			element=element->next;
			continue;
		}
		current_song=current_album->first_song;
		break;
	}

	play_file(current_song);

}


void play_next_song() {

	struct minibus_list_element *element;
	struct s_song *song;

	if (!childpid) {
		killchild();
	}

	element=minibus_find_data(&songlist,current_song);
	if (!element) {
		printf("Song not found\n");
		current_iface=IFACE_MAIN;
		repaint_interface();
		return;
	}

	if (element->next) {
		current_song=element->next->data;
	} else {
		current_iface=IFACE_MAIN;
		repaint_interface();
		return;
	}

	song=element->next->data;
	if ((play_mode==PLAY_TITLES)&&(song->album!=current_album)) {
		element=minibus_find_data(&albumlist,current_album);
		while(1) {
			if (!element) {
				printf("Next album not found\n");
				current_iface=IFACE_MAIN;
				repaint_interface();
				return;
			}
			if (!element->next) {
				current_iface=IFACE_MAIN;
				repaint_interface();
				return;
			}
			current_album=element->next->data;
			if (!current_album->first_song) {
				element=element->next;
				continue;
			}
			current_song=current_album->first_song;
			break;
		}
	}
	play_file(current_song);
}


void play_prev_album() {

	struct minibus_list_element *element;

	element=minibus_find_data(&albumlist,current_song->album);
	while(1) {
		if (!element) {
			printf("Prev album not found\n");
			current_iface=IFACE_MAIN;
			repaint_interface();
			return;
		}
		if (!element->prev) {
			current_iface=IFACE_MAIN;
			repaint_interface();
			return;
		}
		current_album=element->prev->data;
		if (!current_album->first_song) {
			element=element->prev;
			continue;
		}
		current_song=current_album->first_song;
		break;
	}
	play_file(current_song);

}


void play_prev_song() {

	struct minibus_list_element *element;
	struct s_song *song;

	if (!childpid) {
		killchild();
	}

	element=minibus_find_data(&songlist,current_song);
	if (!element) {
		printf("Song not found\n");
		current_iface=IFACE_MAIN;
		repaint_interface();
		return;
	}

	if (element->prev) {
		current_song=element->prev->data;
	} else {
		play_file(current_song);
		return;
	}

	song=element->prev->data;
	if ((play_mode==PLAY_TITLES)&&(song->album!=current_album)) {
		element=minibus_find_data(&albumlist,current_album);
		while(1) {
			if (!element) {
				printf("Prev album not found\n");
				current_iface=IFACE_MAIN;
				repaint_interface();
				return;
			}
			if (!element->prev) {
				current_iface=IFACE_MAIN;
				repaint_interface();
				return;
			}
			current_album=element->prev->data;
			if (!current_album->first_song) {
				element=element->prev;
				continue;
			}
			current_song=current_album->last_song;
			break;
		}
	}
	play_file(current_song);

}


void kill_handler(int value) {

	int pid;
	bool found;

	printf("child exited\n");

	found=false;
	do {
		pid = waitpid(-1,NULL,WNOHANG);
		if (childpid && (pid==childpid)) {
			found=true;
		}
	} while (pid>0);

	if (found) {
		childpid=0;
		if (play_status!=PLAY_STOPPED) {
			play_next_song();
		}
	}
}

void exit_cb(struct ttos_object *master,void *data){

	killchild();

}

int main(int argc,char **argv) {

	struct sigaction sg;
	Cstring tmp_path;
	struct stat status;
	int retval;
	struct timeval tv;

	memset(&sg,0,sizeof(struct sigaction));
	sg.sa_handler=kill_handler;
	sigaction(SIGCHLD,&sg,NULL);

	gettimeofday(&tv,NULL);
	srandom(tv.tv_sec);

	setlocale (LC_ALL, "");
	bindtextdomain ("ttos_player", NULL);
	textdomain ("ttos_player");

	tmp_path=getenv("TTOS_BASE_PATH");
	if ((!tmp_path)||(*tmp_path==0)) {
		tmp_path=getcwd(NULL,0);
	}
	songs_path=minibus_path_join(tmp_path,"music");

	retval=stat(songs_path,&status);

	minibus_list_init(&albumlist,destroy_album,NULL);
	minibus_list_init(&songlist,destroy_song,NULL);

	childpid=0;
	master=ttos_init_system(0,argc,argv);
	tab_id=0;

	if (master==NULL) {
		printf("Can't connect to the queue. Aborting\n");
		exit(1);
	}

	if ((retval==-1)||(!S_ISDIR(status.st_mode))) {
		struct ttos_dialog_button buttons = {TTOS_OK_BUTTON , NULL, NULL, 0};
		ttos_show_dialog(master,_("There is no MUSIC folder. Aborting."),TTOS_WIDGET_FONT_ALIGN_CENTER,&buttons,1);
		exit(0);
	}

	master->exit_cb=exit_cb;

	minibus_add_callback(master->queue,10,shuffle_songs_callback,NULL);
	minibus_add_callback(master->queue,11,shuffle_albums_callback,NULL);
	minibus_add_callback(master->queue,12,play_song_callback,NULL);
	minibus_add_callback(master->queue,13,exit_callback,NULL);

	minibus_add_callback(master->queue,20,paint_main_menu_callback,NULL);
	minibus_add_callback(master->queue,21,play_pause_callback,NULL);

	minibus_add_callback(master->queue,22,prev_song_callback,NULL);
	minibus_add_callback(master->queue,23,next_song_callback,NULL);

	minibus_add_callback(master->queue,24,prev_album_callback,NULL);
	minibus_add_callback(master->queue,25,next_album_callback,NULL);

	current_iface=IFACE_MAIN;
	repaint_interface();

	minibus_run_loop(master->queue);

	minibus_remove_callback(master->queue,10);
	minibus_remove_callback(master->queue,11);
	minibus_remove_callback(master->queue,12);
	minibus_remove_callback(master->queue,13);
	minibus_remove_callback(master->queue,20);
	minibus_remove_callback(master->queue,21);
	minibus_remove_callback(master->queue,22);
	minibus_remove_callback(master->queue,23);
	minibus_remove_callback(master->queue,24);
	minibus_remove_callback(master->queue,25);
	ttos_destroy_system(master);
	free(songs_path);

	return 0;
}
