/*
 * Copyright 2011 Raster Software Vigo
 *
 * This file is part of ttos
 *
 * ttos is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * ttos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * tabs.h
 *
 *  Created on: 21/09/2010
 *      Author: raster
 */

#ifndef TABS_H_
#define TABS_H_

#include "global_vars.h"

/**
 * This structure defines a TAB, which is the base for each program "window"
 */

struct _tab {

	uint16_t tab_id; ///< The tab unique identifier
	uint16_t widget_id_counter; ///< The counter to assign to the widgets an ID
	struct _tab *swap_tab;
	struct _client_data *client; ///< The client which owns this tab
	struct minibus_list widgets; ///< List with all the widgets contained in this tab
	struct ttos_color color;
	int16_t x_offset; ///< X offset for widgets
	int16_t y_offset; ///< Y offset for widgets
	int16_t xtmp; ///< Temporary X offset during scrolling
	int16_t ytmp; ///< Temporary Y offset during scrolling
	bool valid_tmp; ///< If true, valid values are stored in xtmp and ytmp
	int16_t xswap; ///< swap point during tab swap
	int16_t yswap; ///< swap point during tab swap
	int16_t minx; ///< Minimum X value
	int16_t miny; ///< Minimum Y value
	int16_t maxx; ///< Maximum X value
	int16_t maxy; ///< Maximum Y value
	uint16_t margin_up; ///< upper margin for widgets
	uint16_t margin_down; ///< lower margin for widgets
	uint16_t margin_left; ///< left margin for widgets
	uint16_t margin_right; ///> right margin for widgets
	bool sensitive; ///< false during autoscroll
	bool to_delete; ///< True if this tab must be deleted after being replaced by other tab
};

struct _tab_scroll {
	struct _tab *tab;
	bool swap_tab;
	uint32_t swap_mode;
	uint32_t event_port;
	uint32_t flags;
	int16_t original_x;
	int16_t original_y;
	int16_t final_x;
	int16_t final_y;
	int16_t current_x;
	int16_t current_y;
};

uint32_t tabs_create_tab(uint32_t port, uint32_t answer_port, struct minibus_connection *connection, void *private_data);
uint32_t tabs_get_current_tab(uint32_t port, uint32_t answer_port, struct minibus_connection *connection, void *private_data);
uint32_t tabs_destroy_tab(uint32_t port, uint32_t answer_port, struct minibus_connection *connection, void *private_data);
void tabs_free_tab(void *tab);
bool tabs_select_edit_tab(struct minibus_connection *connection, uint16_t tab_id);
uint32_t tabs_select_show_tab(uint32_t port, uint32_t answer_port, struct minibus_connection *connection, void *private_data);
void tabs_paint_tab(struct _tab *tab);//, int16_t x, int16_t y, uint16_t w, uint16_t h);
uint32_t tabs_set_bg_color(uint32_t port, uint32_t answer_port, struct minibus_connection *connection, void *private_data);
void tabs_set_minmax(struct _tab *tab,struct _widget *widget);
uint32_t tabs_set_size(uint32_t port, uint32_t answer_port, struct minibus_connection *connection, void *private_data);
uint32_t tabs_set_margins(uint32_t port, uint32_t answer_port, struct minibus_connection *connection, void *private_data);

struct _tab_scroll *tabs_init_tab_scroll(struct _tab *tab, struct _tab *old_tab, int16_t new_x, int16_t new_y, uint32_t event_port,uint32_t flags, bool swap);
void tabs_free_tab_scroll(void *element);
uint32_t tabs_set_tab_offset(uint32_t port, uint32_t answer_port, struct minibus_connection *connection, void *private_data);
void tabs_do_scroll();


#endif /* TABS_H_ */
