/*
 * Copyright 2011 Raster Software Vigo
 *
 * This file is part of ttos
 *
 * ttos is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * ttos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * ttos_keyboard.c
 *
 * helper_functions.h
 *
 *  Created on: 25/01/2011
 *      Author: raster
 */

#ifndef HELPER_FUNCTIONS_H_
#define HELPER_FUNCTIONS_H_

#include "../minibus/minibus.h"
#include "ttos.h"

#define TTOS_MAX_KEY_SIZE 256

struct _ttos_select_file_object {

	bool folder;
	uint32_t widget_uid;
	Cstring name;
};

struct _ttos_key_buttons {
	enum ttos_stock_button stock_button; ///< Button to paint here
	Cstring string; ///< The string inside the button if stock_button is TTOS_NO_STOCK_BUTTON
	Cstring icon; ///< Icon for the button if stock_button is TTOS_NO_STOCK_BUTTON (NULL for no picture)
	bool text; ///< If true, show the text in the button
	uint32_t event; ///< The event number to send when pressed
	uint8_t color; ///< The desired color (from the predefined ones in the 'ttos_keyboard' structure
	uint8_t size; ///< The size: 0=normal; 1=wide; 2=tall; 3=big. Must put an empty button (TTOS_NO_STOCK_BUTTON and string and icon to NULL) in the other positions covered by this button
};

struct ttos_keyboard {
	struct _ttos_key_buttons *buttons; ///< An array of buttons
	struct ttos_color *colors; ///< An array of colors to use with the buttons (the uint8_t color value)
	uint32_t width;  ///< number of colums or button width (function specific)
	uint32_t height; ///< number of rows or button height (function specific)
};

struct ttos_registry_element {

	Cstring key;
	struct minibus_data_element value;

};

struct ttos_registry {

	Cstring name;
	struct minibus_list elements;

};

uint32_t ttos_main_callback(uint32_t port, uint32_t answer_port, struct minibus_connection *connection,void *user_data);
uint32_t ttos_wait_for_error(struct ttos_object *object, uint32_t port);
uint32_t ttos_wait_for_error_2(struct ttos_object *object, uint32_t port,uint32_t *error);
Cstring ttos_select_file(struct ttos_object *master, Cstring base_path, uint32_t flags, enum ttos_file_type *filetypes);
int32_t ttos_show_dialog(struct ttos_object *master, Cstring text, uint32_t text_flags, struct ttos_dialog_button *buttons, uint16_t count);
int32_t ttos_launch_child(Cstring command[], int *pipes);
uint32_t *ttos_paint_keyboard(struct ttos_object *master, uint32_t tab, int16_t x, int16_t y, int16_t width, int16_t height,
		int16_t h_margin, int16_t v_margin, struct ttos_keyboard *keyboard, bool return_uids);
uint32_t *ttos_paint_grid_buttons(struct ttos_object *master, uint32_t tab, int16_t margin, bool vertical, uint16_t elements,
		struct ttos_keyboard *keyboard, bool return_uids);

void ttos_print_type(struct minibus_data_element *data);
void ttos_print_event(uint32_t event);
void ttos_print_error(enum TTOS_SYSTEM_ERRORS error,bool print_no_error);

struct ttos_registry *ttos_new_registry(Cstring name);
void ttos_destroy_registry(struct ttos_registry *object);
int32_t ttos_reload_registry(struct ttos_registry *object);
int32_t ttos_registry_insert_key(struct ttos_registry *registry, Cstring key, struct minibus_data_element *value);
struct ttos_registry_element * ttos_registry_retrieve_key(struct ttos_registry *registry, Cstring key);
int32_t ttos_registry_remove_key(struct ttos_registry *registry, Cstring key);
int32_t ttos_store_registry(struct ttos_registry *object);

#endif /* HELPER_FUNCTIONS_H_ */
