/*
 * Copyright 2011 Raster Software Vigo
 *
 * This file is part of ttos
 *
 * ttos is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * ttos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * ttos_path.c
 *
 *  Created on: 14/01/2011
 *      Author: raster
 */

/** @file */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "ttos.h"

/**
 * Returns the mime type of a file, based in its extension
 *
 * @param fname Filename to check
 *
 * @return The mime type
 */

enum ttos_file_type ttos_check_extension(Cstring fname) {

	uint32_t size_f,size_e;
	struct {
		Cstring extension;
		enum ttos_file_type type;
	} *element,types[] = {{".mp3",TTOS_FTYPE_MP3},
						  {".ogg",TTOS_FTYPE_OGG},
						  {".oga",TTOS_FTYPE_VORBIS},
						  {".app",TTOS_FTYPE_APP},
						  {NULL,TTOS_FTYPE_UNKNOWN}};

	size_f=strlen(fname);

	for (element=types;element->extension;element++) {
		size_e=strlen(element->extension);
		if (size_f<size_e) {
			continue;
		}
		if (!strcasecmp(fname+size_f-size_e,element->extension)) {
			return element->type;
		}
	}
	return (TTOS_FTYPE_UNKNOWN);
}


/**
 * Returns the parent path and, optionally, the tail of the path
 *
 * Example:
 * input: path="/home/path/folder"
 * output: path="/home/path"; returns "folder"
 *
 * @param path The path from which get the upper path
 *
 * @return The tail of the path, or NULL if can't go upper
 */

Cstring ttos_get_upper_path(Cstring path, Cstring *tail) {

	uint32_t size,pos;
	Cstring parent,pointer;

	if(tail) {
		*tail=NULL;
	}

	if (!path) {
		return NULL;
	}

	size=strlen(path);
	if (!size) {
		return(NULL);
	}
	if (!strcmp(path,"/")) {
		return(strdup("/"));
	}
	parent=strdup(path);
	if (parent[size-1]=='/') {
		parent[size-1]=0; // remove final '/'
		size--;
	}

	for(pointer=parent+size,pos=size;pointer!=parent;pointer--,pos--) {
		if (*pointer=='/') {
			*pointer=0;
			break;
		}
	}

	if (pointer!=parent) {
		if (tail) {
			*tail=path+pos+1;
		}
		return(parent);
	}

	if(*parent=='/') {
		*(parent+1)=0;
		if (tail) {
			*tail=path+1;
		}
	} else {
		if (tail) {
			*tail=path;
		}
	}
	return(parent);
}
